% going over all methods, categories and repeition runs 

load('Split/categoryInfo.mat','categories');
load('Split/Natural/split.mat','negTestImages');
nC = length(categories);
overwrite = true;
className = 'ActiveBasisInvariance';
displayModeIsOn = true;

%featureConvolution('GaborS1',0.7,overwrite);
%featureConvolution('GaborS1_11',0.4,overwrite);
%featureConvolution('GaborS1_25',1,overwrite);

selected_runs = [21];

cd Code
mex Ctransform.c
mex Cmax.c
mex Chistog.c
mex mexc_SharedSketch.cpp
mex mexc_ComputeSUM2.cpp
mex mexc_ComputeMAX1.cpp
mex mexc_Copy.cpp
mex mexc_TemplateAffineTransform.cpp
mex mexc_CropInstanceNew.cpp
cd ..

% natural stat
% tilt_GaborS1

%
% learning
%
%{
for iCat = 1:nC
     for iRun = selected_runs
         %
         % learn a template
         %
         experiment = ActiveBasisInvariant(iRun,iCat,categories);
         experiment.learn; % calls the class method learn(this) of ActiveBasisLocalNorm 
         clear experiment
     end
end
%}

%
% visualize the templates
%
for iRun = selected_runs
    % sketch template
	symbolicTemplates = cell(1,nC);
	nSketchToDisplay = 50;
	for iCat = 1:nC
		load(sprintf('Result/%s/%s/model_run%d.mat', categories{iCat}, className, iRun), 'sym',...
			'Mx','My','Mi','Mm');
		sym = displayGaborTemplate(size(sym), Mx(1:nSketchToDisplay),...
			My(1:nSketchToDisplay), Mi(1:nSketchToDisplay), Mm(1:nSketchToDisplay) );
		symbolicTemplates{iCat} = double(sym);
	end
	templates_sym = displayImages(symbolicTemplates,20,100,100);
	if ~exist(sprintf('Result/summary/%s',className), 'dir')
		mkdir(sprintf('Result/summary/%s',className));
	end
	imwrite(templates_sym,sprintf('%s/templates_run%d.png',sprintf('Result/summary/%s',className), iRun));
	sketch_sym = symbolicTemplates;
    
    % texture template
    symbolicTemplates = cell(1,nC);
	nTextureToDisplay = 30;
	for iCat = 1:nC
		load(sprintf('Result/%s/%s/model_run%d.mat', categories{iCat}, className, iRun), 'S2TemplateTexture'...
			);
		Mx = S2TemplateTexture.selectedRow(1:min(end,nTextureToDisplay)) + 1 + floor(size(sym,1)/2);
		My = S2TemplateTexture.selectedCol(1:min(end,nTextureToDisplay)) + 1 + floor(size(sym,2)/2);
		Mi = S2TemplateTexture.selectedOri(1:min(end,nTextureToDisplay));
		sym = displayGaborTemplate(size(sym), Mx,...
			My,...
            Mi,...
            ones(1,length(Mx))...
            );
		symbolicTemplates{iCat} = double(sym);
	end
	templates_sym = displayImages(symbolicTemplates,20,100,100);
	if ~exist(sprintf('Result/summary/%s',className), 'dir')
		mkdir(sprintf('Result/summary/%s',className));
	end
	imwrite(templates_sym,sprintf('%s/templatesTex_run%d.png',sprintf('Result/summary/%s',className), iRun));
	texture_sym = symbolicTemplates;
	
	% mixed templates
	mixed_sym = cell(1,nC);
	for ii = 1:numel(sketch_sym)
		mixedSym = 255*ones(size(sketch_sym{1},1), size(sketch_sym{1},2), 3);
		
		basisSym = double(sketch_sym{ii}-255)/255 * 240;
		texSym = double(texture_sym{ii}-255)/255 * 240;
		mixedSym(:,:,2) = mixedSym(:,:,2) + basisSym ;
		mixedSym(:,:,1) = mixedSym(:,:,1) + basisSym ;
		mixedSym(:,:,3) = mixedSym(:,:,3) + basisSym ;
		mixedSym(:,:,2) = mixedSym(:,:,2) + 0.5*texSym;
		mixedSym(:,:,3) = mixedSym(:,:,3) + 0.5*texSym;
		mixed_sym{ii} = mixedSym;
	end
	templates_sym = displayImagesColor(mixed_sym(:),nC,150,150,false);
	if ~exist(sprintf('Result/summary/%s',className), 'dir')
		mkdir(sprintf('Result/summary/%s',className));
	end
	imwrite(templates_sym,sprintf('%s/templatesMixed_run%d.png',sprintf('Result/summary/%s',className), iRun));
end


%{
% 
% validation on training data: to obtain p(label|template)
%
iCat = 1;
for iRun = selected_runs
    experiment = ActiveBasisInvariant(iRun,iCat,categories);
    experiment.test_on_train_multiclass;
    clear experiment
end

% 
% testing
%
for iRun = selected_runs
    experiment = ActiveBasisInvariant(iRun,1,categories);
    experiment.test_multiclass;
    clear experiment
end
%}

%
% post processing by Bayesian voting
%
%{
for iRun = selected_runs
    experiment = ActiveBasisInvariant(iRun,1,categories);
    experiment.post_process;
    clear experiment
end
%}


%
% post processing by SVM
%

for iRun = selected_runs
    experiment = ActiveBasisInvariant(iRun,1,categories);
    experiment.post_process_svm;
    clear experiment
end

%}

%{

%
% evaluating
%
for iRun = selected_runs
     numImage = 0;
     for iCat = 1:nC
         load(sprintf('Split/%s/split%d.mat',categories{iCat},iRun),'posTestImages');
         pos{iCat} = numImage + (1 : length(posTestImages));
         numImage = numImage + length(posTestImages);
     end
     
     tssMat = zeros(numImage,nC,'single');
     offset = 0;
     for iCat = 1:nC
         load(sprintf('Result/%s/%s/out_run%d.mat', categories{iCat}, className, iRun),'scores');
%          load(sprintf('Result/%s/%s/out_run%d.mat', categories{iCat}, className, iRun),'post_label_given_image','scores');
%          scores = post_label_given_image;
         for i = 1:length(pos{iCat})
             tssMat(offset+i,:) = scores{i}(1,1:nC);
         end
         offset = offset + length(pos{iCat});
     end
     
     [averageRate confusionMatrix correctRate] = getConfusionMatrix( tssMat, pos );

	if ~exist(sprintf('Result/summary/%s',className), 'dir')
		mkdir(sprintf('Result/summary/%s',className));
	end
	save(sprintf('%s/eval_result_run%d.mat',sprintf('Result/summary/%s',className), iRun), 'averageRate', 'confusionMatrix', 'correctRate', 'tssMat');
	
	if displayModeIsOn
		cm_pdf = sprintf( 'Result/summary/%s/confMat_raw_run%d.pdf', className, iRun );
		cm = confusionMatrix;
		for i = 1:size(cm,1)
			cm(i,:) = cm(i,:) / sum(cm(i,:));
		end
		h = figure;
		confMatrixShow(cm,categories,{'FontSize',10}, [], 0)
		print( h, '-dpdf', cm_pdf );
        close(h);
	end
end

%
% evaluating (svm result)
%

for iRun = selected_runs
     numImage = 0;
     for iCat = 1:nC
         load(sprintf('Split/%s/split%d.mat',categories{iCat},iRun),'posTestImages');
         pos{iCat} = numImage + (1 : length(posTestImages));
         numImage = numImage + length(posTestImages);
     end
     
     load(sprintf('%s/svmresult_run%d.mat',sprintf('Result/summary/%s',className), iRun), 'svm_prediction');
     
     
     [averageRate confusionMatrix correctRate] = getConfusionMatrix(svm_prediction,pos);

	if ~exist(sprintf('Result/summary/%s',className), 'dir')
		mkdir(sprintf('Result/summary/%s',className));
	end
	save(sprintf('%s/svm_eval_result_run%d.mat',sprintf('Result/summary/%s',className), iRun), 'averageRate', 'confusionMatrix', 'correctRate', 'svm_prediction');
	
	if displayModeIsOn
		cm_pdf = sprintf( 'Result/summary/%s/confMat_svm_run%d.pdf', className, iRun );
		cm = confusionMatrix;
		for i = 1:size(cm,1)
			cm(i,:) = cm(i,:) / sum(cm(i,:));
		end
		h = figure;
		confMatrixShow(cm,categories,{'FontSize',10}, [], 0)
		print( h, '-dpdf', cm_pdf );
        close(h);
	end
end

%}

