#include "mex.h"
#include "mat.h"
#include "matrix.h"
#include <stdio.h>
#include <string.h> /* for memcpy() */
#include "inscribe.h"
#include "Cmax.h"

extern void inscribe(int windowWidth, int windowHeight, float* map, int width, int height, float* rmap, float val );
extern void cmax(int sx, int sy, float **fI, float **rmap, int Lrange, int Orange, int sub1, int N) ;

char featureVersion[200];
char folder[200];
int iRun;
int Lrange, Orange;
int sub;
float** scores;
float* gainThresholds;
int nGainThresholds;
int windowSize;

/* compute matching scores of all templates */
void computeScores()
{
    char splitfile[200], featurefile[200], catfile[200],
        modelfile[200], categoryName[200], outfile[200],
        imagename[200];
    int nOri;
    MATFile *pmat;
    mxArray *posTestImages, *pS1, *pM1, *categories, *pScores;
    mxArray *tmp;
    int nImage, i, o, nCat, iC, buflen, status, j;
    float **selectedXs, **selectedYs, **selectedOs;
    float **selectedLambdas, **selectedLogZs, **selectedGains;
    int *numFeature;
    float **newM1, **S1, **M1;
    int width, height;
    float score, r;
    int x, y;
    int currentIndThres;
    mwSize dims[2], dims2[2];
    float* start_of_pr;
    int bytes_to_copy;
    
    /* =============================================
       load category info
       =============================================
     */
    sprintf(catfile, "Split/categoryInfo.mat");
    pmat = matOpen(catfile, "r");
    if (pmat == NULL) {
        printf("Error opening file %s\n", catfile);
        printf("(Does the file exist?)\n");
        mexErrMsgTxt("exit..");
    }
    categories = matGetVariable(pmat, "categories");
    nCat = (int)mxGetScalar(matGetVariable(pmat, "nCat"));
    matClose(pmat);
    
    selectedXs = mxCalloc(nCat, sizeof(float*));
    selectedYs = mxCalloc(nCat, sizeof(float*));
    selectedOs = mxCalloc(nCat, sizeof(float*));
    selectedLambdas = mxCalloc(nCat, sizeof(float*));
    selectedLogZs = mxCalloc(nCat, sizeof(float*));
    selectedGains = mxCalloc(nCat, sizeof(float*));
    numFeature = mxCalloc(nCat, sizeof(int));
    
    
    /* =============================================
     * load all templates
     * =============================================
     */
    for( iC = 0; iC < nCat; ++iC )
    {
        tmp = mxGetCell(categories, iC);
        buflen = (mxGetM(tmp) * mxGetN(tmp)) + 1;
        status = mxGetString(tmp, categoryName, buflen);
        if(status != 0) 
        {
            mexWarnMsgTxt("Not enough space. String is truncated.");
        }
        sprintf(modelfile, "Result/%s/ActiveBasisLocalNorm/model_run%d.mat", categoryName, iRun);
        pmat = matOpen(modelfile, "r");
        if( pmat == NULL )
        {
            mexPrintf("%s : Error loading .mat file. Line %d\n", __FILE__, __LINE__); 
        }
        tmp = matGetVariable(pmat, "Mx");
        selectedXs[iC] = (float*)mxGetPr(tmp);
        numFeature[iC] = mxGetM(tmp) * mxGetN(tmp);
        for( j = 0; j < numFeature[iC]; ++j )
        {
            -- (selectedXs[iC][j]); /* make sure that the index starts from 0 */
        }
        tmp = matGetVariable(pmat, "My");
        selectedYs[iC] = (float*)mxGetPr(tmp);
        for( j = 0; j < numFeature[iC]; ++j )
        {
            -- (selectedYs[iC][j]); /* make sure that the index starts from 0 */
        }
        tmp = matGetVariable(pmat, "Mi");
        selectedOs[iC] = (float*)mxGetPr(tmp);

        tmp = matGetVariable(pmat, "Mm");
        selectedLambdas[iC] = (float*)mxGetPr(tmp);
        tmp = matGetVariable(pmat, "Mm1");
        selectedLogZs[iC] = (float*)mxGetPr(tmp);
        tmp = matGetVariable(pmat, "gain");
        selectedGains[iC] = (float*)mxGetPr(tmp);
        
        /* load the parameters for shape deformation */
        if( iC == 0 ) /* enough to read only once */
        {
            tmp = matGetVariable(pmat, "Lrange");
            if( tmp != NULL )
            {
                Lrange = (int)mxGetScalar(tmp);
            }
            else
            {
                Lrange = 3;
            }
            
            tmp = matGetVariable(pmat, "Orange");
            if( tmp != NULL )
            {
                Orange = (int)mxGetScalar(tmp);
            }
            else
            {
                Orange = 1;
            }
            
            tmp = matGetVariable(pmat, "sub");
            sub = (int)mxGetScalar(tmp);
            if( tmp != NULL )
            {
                sub = (int)mxGetScalar(tmp);
            }
            else
            {
                sub = 2;
            }
        }
        matClose(pmat);
    }

    
    /* =============================================
     * load image names 
     * =============================================
     */
    sprintf(splitfile, "Split/%s/split%d.mat",folder,iRun);
    pmat = matOpen(splitfile, "r");
    posTestImages = matGetVariable(pmat, "posTestImages");
    matClose(pmat);
    nImage = mxGetM(posTestImages) * mxGetN(posTestImages);
    scores = mxCalloc(nImage,sizeof(*scores));
    
    /* ==========================================================================================
     * Computing scores for each image, each template and each gain threshold
     * ==========================================================================================
     */
    for( i = 0; i < nImage; ++i )
    {
        /* load image feature */
        tmp = mxGetCell(posTestImages, i);
        if ( tmp == NULL )
        {
        	mexErrMsgTxt("NULL pointer for posTestImages");
        }
        buflen = (mxGetM(tmp) * mxGetN(tmp)) + 1;
        status = mxGetString(tmp, imagename, buflen);
        if (status != 0)
        {
           printf("%s : Error reading string. Line %d\n", __FILE__, __LINE__); 
        }
        sprintf(featurefile, "Feature/%s/%s/%s.mat", featureVersion, folder, imagename);
        pmat = matOpen(featurefile, "r");
        if (pmat == NULL) {
            printf("Error opening file %s\n", featurefile);
            printf("(Does the file exist?)\n");
            mexErrMsgTxt("exit..");
        }
        
        
        pS1 = matGetVariable(pmat, "S1LocalNorm");
        if ( pS1 == NULL )
        {
        	mexPrintf( "reading: %s\n", featurefile );
        	mexErrMsgTxt( "S1LocalNorm does not exist!" );
        }
        nOri = mxGetM(pS1) * mxGetN(pS1);
       
        
        S1 = mxCalloc(nOri, sizeof(*S1));
        for( o = 0; o < nOri; ++o )
        {
            tmp = mxGetCell(pS1, o);
            S1[o] = (float*)mxGetPr(tmp);
            width = mxGetN(tmp); height = mxGetM(tmp);
        }
        
        /* lcoal maximization */
        M1 = mxCalloc(nOri, sizeof(*M1));
        for( o = 0; o < nOri; ++o )
        {
            M1[o] = (float*)mxCalloc(width*height, sizeof(**M1));
            for( x = 0; x < width * height; ++x )
            {
                M1[o][x] = 0;
            }
        }
        cmax(height,width,S1,M1,Lrange,Orange,sub,nOri);
        
        /* inscribe the M1 map to the square template */
        newM1 = mxCalloc(nOri, sizeof(*newM1));
        for( o = 0; o < nOri; ++o )
        {
            newM1[o] = (float*)mxCalloc(windowSize*windowSize, sizeof(**newM1));
            inscribe(windowSize, windowSize, newM1[o], width, height, M1[o],  0);
        }
       
        /* compute scores */
        /* Note: the features should have already been sorted by their gains. */
        scores[i] = (float*)mxCalloc(nCat*nGainThresholds,sizeof(**scores));
        for( iC = 0; iC < nCat; ++iC )
        {
            score = 0;
            currentIndThres = 0;
            for( j = 0; j < numFeature[iC]; ++j )
            {
                x = (int)selectedXs[iC][j];
                y = (int)selectedYs[iC][j];
                o = (int)selectedOs[iC][j];
                
                r = newM1[o][y * windowSize + x];
                
                /* The following IF branch means:
                 * 
                 * features 0,1,...,j-1 are all above gainThresholds[currentIndThres]
                 *
                 */
                if( selectedGains[iC][j] < gainThresholds[currentIndThres] )
                {
                    /*
                     * scores[i][iC*nGainThresholds+currentIndThres] is 
                     *     the sum of individual scores for features with
                     *     gains above gainThresholds[currentIndThres].
                     */
                      
                    scores[i][iC*nGainThresholds+currentIndThres] = score;
                    ++currentIndThres;
                    if( currentIndThres >= nGainThresholds )
                    {
                        /* rest of the features are all below the lowerst threshold */
                        break;
                    }
                }
                score += r * selectedLambdas[iC][j] - selectedLogZs[iC][j];   
            }
            
            
            
            /* fill in the rest of scores if needed; sometimes the thresholds may be too low */
            while( currentIndThres < nGainThresholds )
            {
                scores[i][iC*nGainThresholds+currentIndThres] = score;
                ++currentIndThres;
            }
            
           
            
        }
        matClose(pmat);
    }

    
    
    /* =============================================
     * Save the scores to out_run%d.mat
     * =============================================
     */
    
    dims[0] = nImage; dims[1] = 1;
    
    pScores = mxCreateCellArray(2, dims);
    for (i=0; i<nImage; ++i)
    {
        dims2[0] = nGainThresholds; dims2[1] = nCat;
        tmp = mxCreateNumericArray(2, dims2 ,mxSINGLE_CLASS, mxREAL);
        start_of_pr = (float*)mxGetData(tmp);
        bytes_to_copy = nCat * nGainThresholds * mxGetElementSize(tmp);
        for( x = 0; x < nCat * nGainThresholds; ++x )
        {
            start_of_pr[x] = scores[i][x];
        }
        /* memcpy(start_of_pr,scores[i],bytes_to_copy); */
        mxSetCell(pScores, i, tmp);
    }
        
    sprintf(outfile, "Result/%s/ActiveBasisLocalNorm/out_run%d.mat", folder, iRun);
    pmat = matOpen(outfile, "w");
    status = matPutVariable(pmat, "scores", pScores);
    if (status != 0) 
    {
        mexWarnMsgTxt("Warning:");
        printf("%s :  Error using matPutVariable on line %d\n", __FILE__, __LINE__);
    }
    if (matClose(pmat) != 0) {
      printf("Error closing file %s\n",outfile);
  }
}

void mexFunction(int nlhs, mxArray *plhs[],
                 int nrhs, const mxArray *prhs[])                
{
    int buflen;
    int status;
    
    /*
	 * input variable 0: featureVersion
	 */
    /* Get the length of the input string. */
    buflen = (mxGetM(prhs[0]) * mxGetN(prhs[0])) + 1;
    status = mxGetString(prhs[0], featureVersion, buflen);
    if (status != 0)
      mexWarnMsgTxt("Not enough space. String is truncated.");

    /*
	 * input variable 1: folder
	 */
    /* Get the length of the input string. */
    buflen = (mxGetM(prhs[1]) * mxGetN(prhs[1])) + 1;
    status = mxGetString(prhs[1], folder, buflen);
    if (status != 0)
      mexWarnMsgTxt("Not enough space. String is truncated.");
    
    /*
	 * input variable 2: iRun
	 */
    iRun = (int)mxGetScalar(prhs[2]);
    
    /*
	 * input variable 3: windowSize
	 */
    windowSize = (int)mxGetScalar(prhs[3]);
    
    /*
	 * input variable 4: gainThresholds
	 */
    gainThresholds = (float*)mxGetPr(prhs[4]);
    nGainThresholds = mxGetM(prhs[4]) * mxGetN(prhs[4]);
    
    
    computeScores();
}
